from __future__ import annotations

from pathlib import Path

import frontmatter

from ....__share__ import Logger
from ...auditing import AuditBuilder
from ..pymd_engine import convert_markdown
from .helpers import audit_template, log_template

__all__ = ["make_toc", "md_context"]

Log = Logger(__name__).Log


def make_toc(index_path: Path) -> list[tuple[str, str]]:
    toc_list = [
        (
            matter.metadata.get("order", -1),
            matter.metadata["title"],
            f"{index_path.parent.stem}-{series_entry.stem}",
        )
        for series_entry in index_path.parent.iterdir()
        if series_entry != index_path
        if not series_entry.is_dir()
        for matter in [frontmatter.load(series_entry)]
    ]
    return sorted(toc_list)


@log_template
@audit_template
def md_context(template, audit_builder: AuditBuilder):
    """A context providing the parsed HTML and scanning it for KaTeX"""
    md_content = frontmatter.load(template.filename)
    html_content = convert_markdown(md_content.content)
    has_katex = """<span class="katex">""" in html_content
    series_toc_magic = "%series_toc%"
    has_series_toc = series_toc_magic in html_content
    extra_flags = {"katex": has_katex}
    if has_series_toc:
        html_content = html_content.replace(series_toc_magic, "")
        series_toc_list = make_toc(index_path=Path(template.filename))
        extra_flags["series_toc"] = series_toc_list
    return {"post_content_html": html_content, **extra_flags}
